@@ -27,6 +27,17 @@ const char *conf_brightness_level = "/apps/asui_state/brightness_level";
 const char *conf_theme = "/apps/asui_state/theme";
 const char *conf_ssh_server = "/apps/asui_state/ssh_server";
 
+const char *conf_alarm_open = "/apps/asui_state/alarm_open";
+const char *conf_alarm_tklock = "/apps/asui_state/alarm_tklock";
+const char *conf_alarm_cb_service = "/apps/asui_state/alarm_cb_service";
+const char *conf_alarm_cb_path = "/apps/asui_state/alarm_cb_path";
+const char *conf_alarm_cb_iface = "/apps/asui_state/alarm_cb_iface";
+const char *conf_alarm_cb_method = "/apps/asui_state/alarm_cb_method";
+const char *conf_alarm_description = "/apps/asui_state/alarm_description";
+const char *conf_alarm_sound = "/apps/asui_state/alarm_sound";
+const char *conf_alarm_mode = "/apps/asui_state/alarm_mode";
+const char *conf_alarm_timestamp = "/apps/asui_state/alarm_timestamp";
+
 GConfClient *gconf_client;
 
 // battery
@@ -45,6 +56,8 @@ unsigned cfg_secure_action, cfg_secure_power_action, cfg_secure_button;
 unsigned cfg_power_unlock_key;
 // user interface
 unsigned cfg_service_size_unit, cfg_short_tap_markers, cfg_start_visible, cfg_threshold_black, cfg_threshold_white;
+// alarm
+unsigned cfg_alarm_duration, cfg_alarm_volume_min, cfg_alarm_volume_max, cfg_alarm_volume_step;
 
 // state
 unsigned cfg_theme, cfg_ssh_server;
@@ -131,6 +144,12 @@ static const char *conf_short_tap_markers = "/apps/asui/short_tap_markers";
 static const char *conf_start_visible = "/apps/asui/start_visible";
 static const char *conf_threshold_black = "/apps/asui/threshold_black";
 static const char *conf_threshold_white = "/apps/asui/threshold_white";
+// alarm
+static const char *conf_alarm_duration = "/apps/asui/alarm_duration";
+static const char *conf_alarm_volume_min = "/apps/asui/alarm_volume_min";
+static const char *conf_alarm_volume_max = "/apps/asui/alarm_volume_max";
+static const char *conf_alarm_volume_step = "/apps/asui/alarm_volume_step";
+
 
 static void make_bool( unsigned *variable, const char *key, unsigned initial_value, unsigned flag ){
 	GError *error = NULL; GConfError code;
@@ -153,6 +172,7 @@ static void make_bool( unsigned *variable, const char *key, unsigned initial_val
 
 #endif
 
+// required by x86 code
 static void make_uint( unsigned *variable, const char *key, unsigned initial_value, unsigned flag ){
 	GError *error = NULL; GConfError code;
 	int value;
@@ -173,6 +193,27 @@ static void make_uint( unsigned *variable, const char *key, unsigned initial_val
 	if(*variable != (unsigned)value){ *variable = value; hw_changes |= flag; }
 }
 
+/*
+static void make_float( float *variable, const char *key, float initial_value, unsigned flag ){
+	GError *error = NULL; GConfError code;
+	float value;
+
+	// test if key exists by querying with wrong type
+	gconf_client_get_bool(gconf_client, key, &error);
+	if(error == NULL) code = GCONF_ERROR_BAD_KEY; else { code = error->code; g_error_free(error); }
+
+	if(code == GCONF_ERROR_TYPE_MISMATCH){
+		// key exists
+		value = gconf_client_get_float(gconf_client, key, NULL);
+	} else {
+		// create the key
+		value = initial_value;
+		conf_set_float(key, value);
+	}
+	if(*variable != value){ *variable = value; hw_changes |= flag; }
+}
+*/
+
 ////////////////////////////////////////////////////////////////////////// BUTTON CELLS
 
 static unsigned load_button( s_button *button, unsigned initial_cell, unsigned force_initial ){
@@ -346,6 +387,11 @@ void config_init( ){
 	gconf_client_unset(gconf_client, conf_threshold_black, NULL);
 	gconf_client_unset(gconf_client, conf_threshold_white, NULL);
 
+	gconf_client_unset(gconf_client, conf_alarm_duration, NULL);
+	gconf_client_unset(gconf_client, conf_alarm_volume_min, NULL);
+	gconf_client_unset(gconf_client, conf_alarm_volume_max, NULL);
+	gconf_client_unset(gconf_client, conf_alarm_volume_step, NULL);
+
 	gconf_client_unset(gconf_client, "/apps/asui", NULL);
 
 	gconf_client_unset(gconf_client, button_lock1.conf, NULL);
@@ -455,6 +501,11 @@ void config_reload( ){
 	make_bool(&cfg_start_visible,			conf_start_visible,				0,		0);					// default: 0	not visible
 	make_uint(&cfg_threshold_black,			conf_threshold_black,			3000,	HW_THEME);			// default: 	to black theme when below 3000 lux
 	make_uint(&cfg_threshold_white,			conf_threshold_white,			4000,	HW_THEME);			// default: 	to white theme when 4000 lux or greater
+	// alarm
+	make_uint(&cfg_alarm_duration,			conf_alarm_duration,			60,		0);					// default:		10-3600 seconds
+	make_uint(&cfg_alarm_volume_min,		conf_alarm_volume_min,			50,		0);					// default:		10-100
+	make_uint(&cfg_alarm_volume_max,		conf_alarm_volume_max,			100,	0);					// default:		10-100
+	make_uint(&cfg_alarm_volume_step,		conf_alarm_volume_step,			125,	0);					// default:		0-1000 milliseconds
 	// state
 	unsigned brightness_level;
 	make_uint(&brightness_level,			conf_brightness_level,			31,		HW_BRIGHTNESS);		// default: 31	25% brightness
@@ -469,6 +520,7 @@ void config_reload( ){
 	// popup window duration
 	if(cfg_popup_duration > 15){ cfg_popup_duration = 5; conf_set_int(conf_popup_duration, 5); }
 	if(cfg_popup_cooldown < 60 || cfg_popup_cooldown > 3600){ cfg_popup_cooldown = 300; conf_set_int(conf_popup_cooldown, 300); }
+	if(cfg_popup_noasui) cfg_popup_onmap = 0;
 
 	// make sure buttons are correctly bound and don't exceed step values
 	if(cfg_buttons_rotate > 3){ cfg_buttons_rotate = 0; conf_set_int(conf_buttons_rotate, 0); } // disable if invalid
@@ -535,6 +587,12 @@ if(cfg_brightness_enhanced && is_770){ cfg_brightness_enhanced = 0; conf_set_boo
 			hw_query_battery(1); // flush
 	}
 
+	// alarm
+	if(cfg_alarm_duration > 3600 || cfg_alarm_duration < 10){ cfg_alarm_duration = 60; conf_set_int(conf_alarm_duration, 60); }
+	if(cfg_alarm_volume_min > 100 || cfg_alarm_volume_min < 10){ cfg_alarm_volume_min = 50; conf_set_int(conf_alarm_volume_min, 50); }
+	if(cfg_alarm_volume_max > 100 || cfg_alarm_volume_max < cfg_alarm_volume_min){ cfg_alarm_volume_max = 100; conf_set_int(conf_alarm_volume_max, 100); }
+	if(cfg_alarm_volume_step > 1000){ cfg_alarm_volume_step = 125; conf_set_int(conf_alarm_volume_step, 125); }
+
 	prev_brightness_enhanced = cfg_brightness_enhanced;
 	prev_battery_idle_time = cfg_battery_idle_time;
 
@@ -608,6 +666,11 @@ if(cfg_brightness_enhanced && is_770){ cfg_brightness_enhanced = 0; conf_set_boo
 	debug_write("  cfg_start_visible = %u", cfg_start_visible);
 	debug_write("  cfg_threshold_black = %u", cfg_threshold_black);
 	debug_write("  cfg_threshold_white = %u", cfg_threshold_white);
+	// alarm
+	debug_write("  cfg_alarm_duration = %u", cfg_alarm_duration);
+	debug_write("  cfg_alarm_volume_min = %u", cfg_alarm_volume_min);
+	debug_write("  cfg_alarm_volume_max = %u", cfg_alarm_volume_max);
+	debug_write("  cfg_alarm_volume_step = %.3f", cfg_alarm_volume_step);
 
 	debug_write("STATE");
 	// state
