@@ -11,6 +11,7 @@
 #endif
 
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
 #ifndef __USE_ISOC99
@@ -26,6 +27,7 @@
 #include "hardware.h"
  #include "dbus.h"
 #include "dsme.h"
+#include "alarm.h"
 
 ////////////////////////////////////////////////////////////////////////// GLOBALS
 
@@ -33,7 +35,6 @@ DBusError dbus_error;
 
 unsigned dbus_listening;
 
-char dbus_wakeup_message;
 int dbus_wakeup_fd;
 
 unsigned original_screen_autolock, original_screen_stayslit;
@@ -49,9 +50,9 @@ static const char *_notification_service_ = "org.freedesktop.Notifications";
 static const char *_notification_path_ = "/org/freedesktop/Notifications";
 static const char *_notification_interface_ = "org.freedesktop.Notifications";
 static const char *_notification_method_ = "SystemNoteInfoprint"; // method
-#endif
 
 static unsigned acting_dead_state;
+#endif
 
 //////////////////////////////////////////////////////////////////////////
 
@@ -86,6 +87,8 @@ static DBusHandlerResult signal_filter( DBusConnection *connection, DBusMessage
 	// needs to be updated here for debug_write() and hw_query_battery()
 	current_time = get_utime();
 
+	dbus_error_init(&dbus_error);
+
 	if(dbus_message_is_method_call(message, "com.qzx.asui", "flush_calls")){
 
 		debug_write("[DBUS] method: flush calls");
@@ -149,9 +152,6 @@ static DBusHandlerResult signal_filter( DBusConnection *connection, DBusMessage
 		dbus_send_reply(signal_bus, message, _INT32_, &response, _END_);
 		// reply to this so MCE will change state to locked and send out signal
 
-		if(hw_screen.locked)
-			mce_lock_screen(0);
-
 		return DBUS_HANDLER_RESULT_HANDLED;
 
     } else if(dbus_message_is_method_call(message, "com.nokia.system_ui.request", "devlock_close")){
@@ -165,43 +165,66 @@ static DBusHandlerResult signal_filter( DBusConnection *connection, DBusMessage
 	////////////////////
 	// ALARM UI
 	} else if(dbus_message_is_method_call(message, "com.nokia.system_ui.request", "alarm_open")){
-		char *cd_service, *cb_path, *cb_iface, *cb_method, *description, *sound, *image, *title;
+		char *cb_service, *cb_path, *cb_iface, *cb_method, *description, *sound, *image, *title;
 		unsigned mode, timestamp;
 
 		debug_write("[DBUS] method: com.nokia.system_ui.request.alarm_open");
 
-		if(dbus_message_get_args(message, &dbus_error, _STRING_, &cd_service, _STRING_, &cb_path, _STRING_, &cb_iface, _STRING_, &cb_method,
+		if(dbus_message_get_args(message, &dbus_error, _STRING_, &cb_service, _STRING_, &cb_path, _STRING_, &cb_iface, _STRING_, &cb_method,
 		   _UINT32_, &mode, _STRING_, &description, _STRING_, &sound, _STRING_, &image, _STRING_, &title, _UINT32_, &timestamp, _END_))
 		{
+			// save to gconf in case ASUI crashes
+			conf_set_string(conf_alarm_cb_service, cb_service);
+			conf_set_string(conf_alarm_cb_path, cb_path);
+			conf_set_string(conf_alarm_cb_iface, cb_iface);
+			conf_set_string(conf_alarm_cb_method, cb_method);
+			conf_set_string(conf_alarm_description, description);
+			conf_set_string(conf_alarm_sound, sound);
+			conf_set_int(conf_alarm_mode, mode);
+			conf_set_int(conf_alarm_timestamp, timestamp);
+			conf_set_bool(conf_alarm_open, 1);
+			conf_set_bool(conf_alarm_tklock, hw_screen.locked);
 
-const char *modes[] = {"normal", "no snooze", "switch on"};
-printf("ALARM: [%s] \"%s\"\n       [PLAY] %s\n", modes[mode], description, sound);
-//foo
+			// reply to the method
+			int response = GTK_RESPONSE_ACCEPT;
+			dbus_send_reply(signal_bus, message, _INT32_, &response, _END_); // or GTK_RESPONSE_REJECT
 
-int response = GTK_RESPONSE_ACCEPT;
-dbus_send_reply(signal_bus, message, _INT32_, &response, _END_); // or GTK_RESPONSE_REJECT
+			// screen will be turned on if it was off
+			dsme_display_state_waiting = 0; // reset blanking pause check
 
+//			if(use_minimal_ui){
+//				hw_unlock_screen();
+//				hw_unlock_keys();
+//				// make sure ASUI thinks screen is locked so alarm_close() will relock it
+//				hw_screen.locked = 1;
+//			}
 
-// dismiss the alarm
-unsigned status = 2; // 1:snooze, 2:dismiss, 3:powerUP, 4:powerDOWN
-dbus_send_method(cd_service, cb_path, cb_iface, cb_method, _INT32_, &status, _END_);
+			// alarmd unlocks screen and keys, unable to relock them with MCE or manually
+			unsigned tklock = hw_screen.locked;
+			if(tklock){
+				hw_screen.locked = 0;
+				notify_screen_unlocked();
+			}
+
+			alarm_open(tklock, cb_service, cb_path, cb_iface, cb_method, description, sound, mode, timestamp);
 
 			return DBUS_HANDLER_RESULT_HANDLED;
 		}
 
 	} else if(dbus_message_is_method_call(message, "com.nokia.system_ui.request", "alarm_close")){
+		// not sure what invokes this method
 
-// remove \n
-printf("[DBUS] method: com.nokia.system_ui.request.alarm_close\n");
+		debug_write("[DBUS] method: com.nokia.system_ui.request.alarm_close");
 
-// TODO: close the alarm popup
+		conf_set_bool(conf_alarm_open, 0);
+		alarm_close(0);
 
 		return DBUS_HANDLER_RESULT_HANDLED;
 
 	} else if(dbus_message_is_method_call(message, "com.nokia.system_ui.request", "acting_dead_open")){
+		// not sure what invokes this method or what it expects
 
-// remove \n
-printf("[DBUS] method: com.nokia.system_ui.request.acting_dead_open\n");
+		debug_write("[DBUS] method: com.nokia.system_ui.request.acting_dead_open");
 
 // TODO: inputs? reply value?
 acting_dead_state = 1;
@@ -209,18 +232,20 @@ acting_dead_state = 1;
 		return DBUS_HANDLER_RESULT_HANDLED;
 
 	} else if(dbus_message_is_method_call(message, "com.nokia.system_ui.request", "acting_dead_close")){
+		// not sure what invokes this method or what it expects
 
-// remove \n
-printf("[DBUS] method: com.nokia.system_ui.request.acting_dead_close\n");
+		debug_write("[DBUS] method: com.nokia.system_ui.request.acting_dead_close");
+
+// TODO:
 acting_dead_state = 0;
 
 		return DBUS_HANDLER_RESULT_HANDLED;
 
 	} else if(dbus_message_is_method_call(message, "com.nokia.system_ui.request", "acring_dead_getstate")){
 
-// remove \n
-printf("[DBUS] method: com.nokia.system_ui.request.acring_dead_getstate\n");
+		debug_write("[DBUS] method: com.nokia.system_ui.request.acring_dead_getstate");
 
+// TODO: this assumes acring_dead_getstate is a typo for acting_dead_state
 		dbus_send_reply(signal_bus, message, _BOOL_, &acting_dead_state, _END_);
 
 		return DBUS_HANDLER_RESULT_HANDLED;
@@ -240,21 +265,23 @@ printf("[DBUS] method: com.nokia.system_ui.request.acring_dead_getstate\n");
 					// this signal will turn on the screen, if off
 					dsme_display_state_waiting = 0; // reset blanking pause check
 
-					if(cfg_power_unlock_key && hw_screen.locked){
-						if(hw_screen.visible && current_time - hw_screen.visible_time > 1.0){
-							// turn screen back off
-							mce_lock_and_blank_screen();
-						} else {
-							// secondary key required to unlock screen, keypad remains unlocked
-							hw_lock_screen();
-							hw_unlock_keys();
-							if(!window.is_mapped) window_open_blank(); // open blank to catch keys
-							notify_power_unlock_key();
-						}
-					} else if(blank_window_open)
-						window_close_blank(); // power key caught, close the blank window
-					else
-						dbus_send_wakeup(_WAKEUP_POWER_BUTTON_);
+					if(!use_minimal_ui){
+						if(cfg_power_unlock_key && hw_screen.locked){
+							if(hw_screen.visible && current_time - hw_screen.visible_time > 1.0){
+								// turn screen back off
+								mce_lock_and_blank_screen();
+							} else {
+								// secondary key required to unlock screen, keypad remains unlocked
+								hw_lock_screen();
+								hw_unlock_keys();
+								if(!window.is_mapped) window_open_blank(); // open blank to catch keys
+								notify_power_unlock_key();
+							}
+						} else if(blank_window_open)
+							window_close_blank(); // power key caught, close the blank window
+						else
+							dbus_send_wakeup(_WAKEUP_POWER_BUTTON_);
+					}
 					return DBUS_HANDLER_RESULT_HANDLED;
 				} else if(!strcmp(s2, "connection")){
 					debug_write("[DBUS]         headphone connection (blank=%u)", blank_window_open);
@@ -318,7 +345,7 @@ printf("[DBUS] method: com.nokia.system_ui.request.acring_dead_getstate\n");
 				}
 				hw_unlock_keys(); // always make sure keypad is unlocked, even if ASUI doesn't think the screen is locked
 				hw_screen.locked = 0;
-//printf("com.nokia.mce.signal.tklock_mode_ind -- screen UNLOCKED\n");
+//printf("DBUS: tklock_mode_ind -- unlocked\n");
 			} else if(!strcmp(s1, "locked")){
 				debug_write("[DBUS] signal: com.nokia.mce.signal.tklock_mode_ind (lock)");
 				if(!hw_screen.locked && hw_screen.visible){
@@ -327,7 +354,7 @@ printf("[DBUS] method: com.nokia.system_ui.request.acring_dead_getstate\n");
 				}
 				hw_lock_keys(); // always make sure keypad is locked, even if ASUI doesn't think the screen is unlocked
 				hw_screen.locked = 1;
-//printf("com.nokia.mce.signal.tklock_mode_ind -- screen LOCKED\n");
+//printf("DBUS: tklock_mode_ind -- LOCKED\n");
 
 				window_close_blank(); // close the blank window, if open
 				click.enabled = 1; // don't ignore first tap since screen can't be unlocked by tapping it
@@ -402,7 +429,7 @@ printf("[DBUS] method: com.nokia.system_ui.request.acring_dead_getstate\n");
 
 		if(dbus_message_get_args(message, &dbus_error, _STRING_, &s1, _END_)){
 			if(!strcmp(s1, "unlocked")){
-//printf("com.nokia.mce.signal.devicelock_mode_ind -- code UNLOCKED\n");
+//printf("DBUS: devicelock_mode_ind -- unlocked\n");
 				if(hw_device_locked){
 					debug_write("[DBUS] signal: com.nokia.mce.signal.devicelock_mode_ind (code unlock)");
 					test_write("signal: unlock device");
@@ -410,7 +437,7 @@ printf("[DBUS] method: com.nokia.system_ui.request.acring_dead_getstate\n");
 					dbus_send_wakeup(_WAKEUP_DEVICE_LOCK_);
 				}
 			} else if(!strcmp(s1, "locked")){
-//printf("com.nokia.mce.signal.devicelock_mode_ind -- code LOCKED\n");
+//printf("DBUS: devicelock_mode_ind -- LOCKED\n");
 				if(!hw_device_locked){
 					debug_write("[DBUS] signal: com.nokia.mce.signal.device_mode_ind (code lock)");
 					test_write("signal: lock device");
@@ -809,6 +836,7 @@ static void *dbus_thread( void *arg ){
 int dbus_init( ){
 	#ifdef ARCH_armel
 	original_screen_autolock = original_screen_stayslit = 0;
+	acting_dead_state = 0;
 
 	dbus_threads_init_default();
 	dbus_error_init(&dbus_error);
@@ -849,6 +877,10 @@ int dbus_init( ){
 	return PARENT_READ;
 }
 
+void dbus_send_wakeup( char message ){
+	write(dbus_wakeup_fd, (void *)&message, 1);
+}
+
 void dbus_disconnect( ){
 	#ifdef ARCH_armel
 	dbus_connection_close(system_bus);
