@@ -374,6 +374,12 @@ void mce_init( ){
 
 ////////////////////
 
+void mce_wake_screen( ){
+	#ifdef ARCH_armel
+	dbus_send_method(_mce_service_, _mce_path_, _mce_interface_, "req_display_state_on", _END_);
+	#endif
+}
+
 void mce_dim_screen( ){
 	#ifdef ARCH_armel
 	if(hw_battery.status == BATTERY_DRAINING || !conf_get_screen_stayslit_value()){
@@ -468,6 +474,18 @@ void hw_lock_screen( ){
 	#endif
 }
 
+void hw_unlock_screen( ){
+	#ifdef ARCH_armel
+	FILE *out = fopen(TOUCHSCREEN_CONTROL, "w");
+	if(out != NULL){
+		fputc('0', out);
+		fclose(out);
+	} else {
+		debug_write("failed to open file needed to unlock screen");
+	}
+	#endif
+}
+
 void hw_lock_keys( ){
 	#ifdef ARCH_armel
 	FILE *out = fopen(keypad_control, "w");
@@ -475,7 +493,7 @@ void hw_lock_keys( ){
 		fputc('1', out);
 		fclose(out);
 	} else {
-		debug_write("failed to open file needed to lock keypad");
+		debug_write("failed to open file needed to lock keys");
 	}
 	#endif
 }
@@ -487,7 +505,7 @@ void hw_unlock_keys( ){
 		fputc('0', out);
 		fclose(out);
 	} else {
-		debug_write("failed to open file needed to unlock keypad");
+		debug_write("failed to open file needed to unlock keys");
 	}
 	#endif
 }
