@@ -55,6 +55,7 @@
 #include "dbus.h"
 #include "dsme.h"
 #include "secure.h"
+#include "alarm.h"
 
 // open flashlight on fullscreen button
 #include "flashlight.h"
@@ -801,7 +802,7 @@ static void highlight_button_bar( ){
 	// this usleep also fixes a problem where X wouldn't update the window contents for long periods of time
 }
 
-static int screen_event( ){
+static void screen_event( ){
 	if(current_dialog_draw){
 
 		if(current_dialog_click && click.timestamp > dialog_throttle.timestamp + DIALOG_THROTTLE_DURATION) current_dialog_click();
@@ -882,8 +883,6 @@ static int screen_event( ){
 		}
 
 	}
-
-	return 0;
 }
 
 ////////////////////////////////////////////////////////////////////////// ROTATE, BRIGHTNESS, VOLUME BUTTONS
@@ -1296,6 +1295,9 @@ if(runlevel == 2){
 	dsme_fd = 0;
 	#endif
 
+	// initialize alarms
+	alarm_init();
+
 	// initialize dbus thread
 	if(!(dbus_fd = dbus_init())){
 		clean_up();
@@ -1322,6 +1324,35 @@ if(runlevel == 2){
 
 	dbus_send_signal("/com/nokia/system_ui/signal", "com.nokia.system_ui.signal", "system_ui_started", _END_);
 
+	if(conf_get_bool(conf_alarm_open)){
+		// ASUI terminated while an alarm was open and screen could be in an unblankable state
+
+		usleep(500000); // sleep 500ms to let dbus acquire initial values
+
+		// make sure screen is on
+		mce_wake_screen();
+
+		unsigned tklock = conf_get_bool(conf_alarm_tklock);
+		if(tklock){
+			resume_alarm = 1; // alarm_close() needs to relock tk
+			hw_unlock_screen();
+			hw_unlock_keys();
+			hw_screen.locked = 0;
+			notify_screen_unlocked();
+		}
+
+		alarm_open(tklock,
+			conf_get_string(conf_alarm_cb_service),
+			conf_get_string(conf_alarm_cb_path),
+			conf_get_string(conf_alarm_cb_iface),
+			conf_get_string(conf_alarm_cb_method),
+			conf_get_string(conf_alarm_description),
+			conf_get_string(conf_alarm_sound),
+			conf_get_int(conf_alarm_mode),
+			conf_get_int(conf_alarm_timestamp)
+		);
+	}
+
 	////////////////////
 
 	int x_fd = ConnectionNumber(window.display);
@@ -1419,7 +1450,7 @@ if(runlevel == 2){
 					click.start_y = click.motion_y = click.y = e.xbutton.y;
 					click.offset_x = click.offset_y = 0; // reset motion
 
-					if(screen_event()) goto exit;
+					screen_event();
 
 					if(button_callback && button_autorepeat)
 						button_repeat_init();
@@ -1440,7 +1471,7 @@ if(runlevel == 2){
 						click.motion_y = e.xmotion.y;
 
 						if(motion_callback == NULL){
-							if(screen_event()) goto exit;
+							screen_event();
 						} else if(is_timer_ready(&motion_timer)){
 							if(motion_callback()) reset_timer(&motion_timer);
 							has_motion = 0;
@@ -1476,7 +1507,7 @@ if(runlevel == 2){
 					}
 
 					reset_long_press();
-					if(screen_event()) goto exit;
+					screen_event();
 
 				} else {
 
@@ -1624,7 +1655,7 @@ if(runlevel == 2){
 
 					// hide the window
 					debug_write("    message HIDE");
-					if(window.is_mapped && !hw_device_locked) window_unmap();
+					if(window.is_mapped && !hw_device_locked && current_dialog_draw != draw_alarm) window_unmap();
 					break;
 
 				case _WAKEUP_ROTATE_LEFT_:
@@ -1663,12 +1694,14 @@ if(runlevel == 2){
 					// this way the power button can be used to wake or unlock the device and it
 					// will be in the same state it was when the screen blanked or locked
 					debug_write("    message POWER BUTTON");
+					// this message is never invoked while in minimal UI
 					if(hw_screen.visible && !hw_screen.locked && current_time - hw_screen.visible_time > 1.0){
-						if(hw_device_locked){
+						if(hw_device_locked && current_dialog_draw != draw_alarm){
+							// device is locked but not in alarm dialog
 							press_lock_button(cfg_secure_power_action+1);
 						} else if(window.is_mapped){
-							// hide the window
-							if(!use_minimal_ui && !hw_device_locked && !cfg_n800_lock_mode)
+							// hide the window unless device locked, n800 lock mode or alarm dialog
+							if(!hw_device_locked && !cfg_n800_lock_mode && current_dialog_draw != draw_alarm)
 								window_unmap();
 						} else
 							window_map();
@@ -1692,10 +1725,20 @@ if(runlevel == 2){
 					// if locked and window is not mapped then map window
 					if(hw_device_locked){
 						debug_write("    message DEVICE LOCK");
-						if(!window.is_mapped)
-							window_map();
-						if(current_dialog_draw != draw_fullsreen_clock && current_dialog_draw != draw_flashlight)
-							secureShowKeypad();
+						if(use_minimal_ui){
+							// when booting from charging runlevel
+							// unlock device and screen and restart ASUI
+							// new ASUI will only display keypad once
+							mce_unlock_device();
+							mce_unlock_screen();
+							usleep(500000); // sleep 500ms
+							goto exit;
+						} else {
+							if(!window.is_mapped)
+								window_map();
+							if(current_dialog_draw != draw_fullsreen_clock && current_dialog_draw != draw_flashlight && current_dialog_draw != draw_alarm)
+								secureShowKeypad();
+						}
 					} else {
 						debug_write("    message DEVICE UNLOCK");
 						secureHideKeypad();
@@ -1779,15 +1822,19 @@ if(runlevel == 2){
 		if(current_dialog_draw != NULL && window.is_mapped && hw_screen.visible)
 			hw_query_als();
 
+		if(activate_alarm) alarm_open_dialog();
+
 		infinite_timeout =
 			((!window.is_mapped && !blank_window_open && !popup_window_open) || !hw_screen.visible || current_dialog_redraw == DLG_REDRAW_OFF ? 1 : 0);
+		next_timer_duration = 0.0;
 
 		// auto refresh when display is lit, window is mapped and dialog has a refresh timer
 		if(!infinite_timeout){
 
 			debug_write("process timers");
 
-			next_timer_duration = (blank_window_open ? 1.0 : 0.0); // poll every second if the blank window is open
+			if(blank_window_open) next_timer_duration = 1.0; // poll every second if the blank window is open
+
 			if(popup_window_open){
 				double popup_remaining_duration = cfg_popup_duration - (current_time - popup_window_open_time);
 				if(popup_remaining_duration <= 0.0)
@@ -1796,11 +1843,12 @@ if(runlevel == 2){
 					next_timer_duration = popup_remaining_duration;
 			}
 
-			// secure dialog is always active when device is code locked (fullscreen clock and flashlight can also be active)
+			// secure dialog is always active when device is code locked (alarm, fullscreen clock and flashlight can also be active)
 			if(hw_device_locked && !use_minimal_ui
 			   && current_dialog_draw != draw_secure_keypad
 			   && current_dialog_draw != draw_fullsreen_clock
 			   && current_dialog_draw != draw_flashlight
+			   && current_dialog_draw != draw_alarm
 			)
 				secureShowKeypad();
 
@@ -1960,8 +2008,11 @@ if(runlevel == 2){
 
 		if(dsme_display_state_waiting){
 			double dsme_remaining = dsme_check_display_state();
-			if(dsme_remaining > 0.0 && dsme_remaining < next_timer_duration)
-				next_timer_duration = dsme_remaining;
+			if(dsme_remaining > 0.0){
+				if(next_timer_duration <= 0.0 || dsme_remaining < next_timer_duration)
+					next_timer_duration = dsme_remaining;
+				infinite_timeout = 0;
+			}
 		}
 
 		// refresh on dbus signals and method returns
