@@ -91,6 +91,9 @@ static void unlock( ){
 	nr_failures++;
 	clear_code();
 	draw_secure_keypad(1); // randomize position on failure
+
+// TODO: ASUI should call the MCE devlock_callback:3 method each time an incorrect code is entered so MCE can update gconf values for failures
+
 }
 
 static void draw_keypad( unsigned force, int x, int y ){
@@ -347,6 +350,8 @@ void secureShowKeypad( ){
 		clear_code();
 		got_code = 0;
 		nr_failures = 0;
+		if(use_minimal_ui)
+			detect_runlevel_transition();
 		dialog_open(draw_secure_keypad, NULL, keypress_secure_keypad, NULL, DLG_REDRAW_ON); // no clicks
 		dsme_query_devicelock_code();
 		set_timer(&dialog_refresh_timer, GETCODE_REFRESH_PERIOD);
